"""DO NOT MODIFY. Auto-generated by script/fingerprint_frontend."""

FINGERPRINTS = {
    "core.js": "adfeb513cf650acf763e284d76a48d6b",
    "frontend.html": "43340b2369646b779e04a9925c225ab4",
    "mdi.html": "c1dde43ccf5667f687c418fc8daf9668",
    "micromarkdown-js.html": "93b5ec4016f0bba585521cf4d18dec1a",
    "panels/ha-panel-dev-event.html": "5c82300b3cf543a92cf4297506e450e7",
    "panels/ha-panel-dev-info.html": "0469024d94d6270a8680df2be44ba916",
    "panels/ha-panel-dev-service.html": "9f749635e518a4ca7991975bdefdb10a",
    "panels/ha-panel-dev-state.html": "7d069ba8fd5379fa8f59858b8c0a7473",
    "panels/ha-panel-dev-template.html": "2b618508510afa5281c9ecae0c3a3dbd",
    "panels/ha-panel-history.html": "8955c1d093a2c417c89ed90dd627c7d3",
    "panels/ha-panel-iframe.html": "d920f0aa3c903680f2f8795e2255daab",
    "panels/ha-panel-logbook.html": "f36297a894524518fa70883f264492b0",
    "panels/ha-panel-map.html": "9c8c7924ba8f731560c9f4093835cc26",
    "websocket_test.html": "575de64b431fe11c3785bf96d7813450"
}
